@@ -151,7 +151,8 @@ class Media(WBValue):
         if extension is not None and not path.endswith(extension):
             raise ValueError('Media file extension "{}" must occur at the end of path "{}".'.format(extension, path))
 
-        self._sha256 = hashlib.sha256(open(self._path, 'rb').read()).hexdigest()
+        with open(self._path, 'rb') as f:
+            self._sha256 = hashlib.sha256(f.read()).hexdigest()
         self._size = os.path.getsize(self._path)
 
     @classmethod
@@ -230,7 +231,7 @@ class Media(WBValue):
             raise RuntimeError(
                 'Value of type {} must be bound to a run with bind_to_run() before being serialized to JSON.'.format(type(self).__name__))
 
-        assert self._run is run, "For now we don't support referring to media files across runs."
+        assert self._run is run, "We don't support referring to media files across runs."
 
         return {
             '_type': 'file',  # TODO(adrian): This isn't (yet) a real media type we support on the frontend.
@@ -262,15 +263,23 @@ class Table(Media):
         columns ([str]): Names of the columns in the table.
             Defaults to ["Input", "Output", "Expected"].
         data (array): 2D Array of values that will be displayed as strings.
+        dataframe (pandas.DataFrame): DataFrame object used to create the table.
+            When set, the other arguments are ignored.
     """
     MAX_ROWS = 10000
 
-    def __init__(self, columns=["Input", "Output", "Expected"], data=None, rows=None):
+    def __init__(self, columns=["Input", "Output", "Expected"], data=None, rows=None, dataframe=None):
         """rows is kept for legacy reasons, we use data to mimic the Pandas api
         """
         super(Table, self).__init__()
         self.columns = columns
         self.data = list(rows or data or [])
+        if dataframe is not None:
+            assert util.is_pandas_data_frame(dataframe), 'dataframe argument expects a `Dataframe` object'
+            self.columns = dataframe.columns.to_list()
+            self.data = []
+            for row in range(len(dataframe)):
+                self.add_data(*tuple(dataframe[col].values[row] for col in self.columns))
 
     def add_row(self, *row):
         logging.warning("add_row is deprecated, use add_data")
@@ -467,10 +476,11 @@ class Object3D(BatchableMedia):
 
             self._set_file(data_or_path, is_tmp=False)
         # Supported different types and scene for 3D scenes
-        elif 'type' in data_or_path:
+        elif isinstance(data_or_path, dict) and 'type' in data_or_path:
             if data_or_path['type'] == 'lidar/beta':
                 data = {
                     'type': data_or_path['type'],
+                    'vectors': data_or_path['vectors'].tolist(),
                     'points': data_or_path['points'].tolist(),
                     'boxes': data_or_path['boxes'].tolist(),
                 }
@@ -496,7 +506,7 @@ class Object3D(BatchableMedia):
                       separators=(',', ':'), sort_keys=True, indent=4)
             self._set_file(tmp_path, is_tmp=True, extension='.pts.json')
         else:
-            raise ValueError("data must be a numpy or a file object")
+            raise ValueError("data must be a numpy array, dict or a file object")
 
     @classmethod
     def get_media_subdir(self):
@@ -517,9 +527,10 @@ class Object3D(BatchableMedia):
         jsons = [obj.to_json(run) for obj in threeD_list]
 
         for obj in jsons:
-            if not obj['path'].startswith(cls.get_media_subdir()):
+            expected = util.to_forward_slash_path(cls.get_media_subdir())
+            if not obj['path'].startswith(expected):
                 raise ValueError('Files in an array of Object3D\'s must be in the {} directory, not {}'.format(
-                    cls.get_media_subdir(), obj['path']))
+                    expected, obj['path']))
 
         return {
             "_type": "object3D",
@@ -1211,9 +1222,9 @@ class ImageMask(Media):
     def __init__(self, val, key, **kwargs):
         super(ImageMask, self).__init__()
 
+        np = util.get_module("numpy", required="Semantic Segmentation mask support requires numpy")
         # Add default class mapping
         if not "class_labels" in val:
-            np = util.get_module("numpy", required="Semantic Segmentation mask support requires numpy")
             classes = np.unique(val["mask_data"]).astype(np.int32).tolist()
             class_labels = dict((c, "class_" + str(c)) for c in classes)
             val["class_labels"] = class_labels
@@ -1227,7 +1238,7 @@ class ImageMask(Media):
 
         PILImage = util.get_module(
             "PIL.Image", required='wandb.Image needs the PIL package. To get it, run "pip install pillow".')
-        image = PILImage.fromarray(val["mask_data"], mode="L")
+        image = PILImage.fromarray(val["mask_data"].astype(np.int8), mode="L")
 
         image.save(tmp_path, transparency=None)
         self._set_file(tmp_path, is_tmp=True, extension=ext)
@@ -1253,6 +1264,7 @@ class ImageMask(Media):
         return "mask"
 
     def validate(self, mask):
+        np = util.get_module("numpy", required="Semantic Segmentation mask support requires numpy")
         # 2D Make this work with all tensor(like) types
         if not "mask_data" in mask:
             raise TypeError("Missing key \"mask_data\": A mask requires mask data(A 2D array representing the predctions)")
@@ -1261,8 +1273,7 @@ class ImageMask(Media):
             shape = mask["mask_data"].shape
             if len(shape) != 2:
                 raise TypeError(error_str)
-            if not ((mask["mask_data"] >= 0).all() and (mask["mask_data"] <= 255).all()) and \
-                    mask["mask_data"].dtype('int8').kind == "i":
+            if not ((mask["mask_data"] >= 0).all() and (mask["mask_data"] <= 255).all()) and issubclass(mask.dtype.type, np.integer):
                 raise TypeError("Mask data must be integers between 0 and 255")
 
         # Optional argument
