@@ -80,7 +80,6 @@ class DataStore(object):
             if os.path.exists(fname):
                 raise IOError("File exists: {}".format(fname))
         self._fp = open(fname, open_flags)
-        # write header
         self._write_header()
 
     def open_for_append(self, fname):
@@ -96,26 +95,56 @@ class DataStore(object):
         self._fp = open(fname, "rb")
         self._index = 0
         self._opened_for_scan = True
+        self._read_header()
 
     def scan_record(self):
         assert self._opened_for_scan
-        # TODO(jhr): if end of block, deal with up to 6 zero bytes
+        # TODO(jhr): handle some assertions as file corruption issues
+        # assume we have enough room to read header, checked by caller?
         header = self._fp.read(LEVELDBLOG_HEADER_LEN)
-        if len(header) != LEVELDBLOG_HEADER_LEN:
+        if len(header) == 0:
             return None
+        assert len(header) == LEVELDBLOG_HEADER_LEN
         fields = struct.unpack("<IHB", header)
         checksum, dlength, dtype = fields
+        # check len, better fit in the block
         data = self._fp.read(dlength)
-        # check len
-        return data
+        checksum_computed = zlib.crc32(data, self._crc[dtype]) & 0xFFFFFFFF
+        assert checksum == checksum_computed
+        self._index += dlength
+        return dtype, data
 
-    def scan_block(self):
-        pass
-
-    def scan(self):
+    def scan_data(self):
+        # TODO(jhr): handle some assertions as file corruption issues
         # how much left in the block.  if less than header len, read as pad,
-        # verify they are zero
-        pass
+        offset = self._index % LEVELDBLOG_BLOCK_LEN
+        space_left = LEVELDBLOG_BLOCK_LEN - offset
+        if space_left < LEVELDBLOG_HEADER_LEN:
+            pad_check = "\x00" * space_left
+            pad = self._fp.read(space_left)
+            # verify they are zero
+            assert pad == pad_check
+            self._index += space_left
+
+        record = self.scan_record()
+        if record is None:  # eof
+            return None
+        dtype, data = record
+        if dtype == LEVELDBLOG_FULL:
+            return data
+
+        assert dtype == LEVELDBLOG_FIRST
+        while True:
+            offset = self._index % LEVELDBLOG_BLOCK_LEN
+            assert offset == 0
+
+            dtype, new_data = self.scan_record()
+            if dtype == LEVELDBLOG_LAST:
+                data += new_data
+                break
+            assert dtype == LEVELDBLOG_MIDDLE
+            data += new_data
+        return None
 
     def _write_header(self):
         data = struct.pack(
@@ -128,6 +157,19 @@ class DataStore(object):
         self._fp.write(data)
         self._index += len(data)
 
+    def _read_header(self):
+        header_length = 7
+        header = self._fp.read(header_length)
+        ident, magic, version = struct.unpack("<4sHB", header)
+        if ident != strtobytes(LEVELDBLOG_HEADER_IDENT):
+            raise Exception("Invalid header")
+        if magic != LEVELDBLOG_HEADER_MAGIC:
+            raise Exception("Invalid header")
+        if version != LEVELDBLOG_HEADER_VERSION:
+            raise Exception("Invalid header")
+        assert len(header) == header_length
+        self._index += len(header)
+
     def _write_record(self, s, dtype=None):
         """Write record that must fit into a block."""
         # double check that there is enough space
@@ -136,7 +178,6 @@ class DataStore(object):
             LEVELDBLOG_BLOCK_LEN - self._index % LEVELDBLOG_BLOCK_LEN
         )
 
-        checksum = 0
         dlength = len(s)
         dtype = dtype or LEVELDBLOG_FULL
         # print("record: length={} type={}".format(dlength, dtype))
