@@ -15,7 +15,9 @@ import shutil
 
 import click
 import wandb
+from wandb.apis import internal, public
 from wandb.data_types import _datatypes_set_callback
+from wandb.util import sentry_set_scope, to_forward_slash_path
 from wandb.viz import Visualize
 
 from . import wandb_config
@@ -64,6 +66,10 @@ class RunManaged(Run):
         # Pull info from settings
         self._init_from_settings(settings)
 
+        # Initial scope setup for sentry. This might get changed when the
+        # actual run comes back.
+        sentry_set_scope("user", self._entity, self._project)
+
         # Returned from backend send_run_sync, set from wandb_init?
         self._run_obj = None
 
@@ -71,8 +77,11 @@ class RunManaged(Run):
         wandb_key = "_wandb"
         config.setdefault(wandb_key, dict())
         config[wandb_key]["cli_version"] = wandb.__version__
-        if config:
-            self._config.update(config)
+        if settings.save_code and settings.code_program:
+            config[wandb_key]["code_path"] = to_forward_slash_path(
+                os.path.join("code", settings.code_program)
+            )
+        self._config._update(config)
 
     def _init_from_settings(self, settings):
         if settings.entity is not None:
@@ -204,8 +213,37 @@ class RunManaged(Run):
 
     def _set_run_obj(self, run_obj):
         self._run_obj = run_obj
+        # TODO: It feels weird to call this twice..
+        sentry_set_scope("user", run_obj.entity, run_obj.project, self._get_run_url())
 
-    def log(self, data, step=None, commit=True):
+    def _add_singleton(self, type, key, value):
+        """Stores a singleton item to wandb config.
+
+        A singleton in this context is a piece of data that is continually
+        logged with the same value in each history step, but represented
+        as a single item in the config.
+
+        We do this to avoid filling up history with a lot of repeated uneccessary data
+
+        Add singleton can be called many times in one run and it will only be
+        updated when the value changes. The last value logged will be the one
+        persisted to the server"""
+        value_extra = {"type": type, "key": key, "value": value}
+
+        if type not in self.config["_wandb"]:
+            self.config["_wandb"][type] = {}
+
+        if type in self.config["_wandb"][type]:
+            old_value = self.config["_wandb"][type][key]
+        else:
+            old_value = None
+
+        if value_extra != old_value:
+            self.config["_wandb"][type][key] = value_extra
+            self.config.persist()
+
+    def log(self, data, step=None, commit=True, sync=None):
+        # TODO(cling): sync is a noop for now
         if commit:
             self.history._row_add(data)
         else:
@@ -349,3 +387,111 @@ class RunManaged(Run):
     def watch(self, models, criterion=None, log="gradients", log_freq=100, idx=None):
         logger.info("Watching")
         # wandb.run.watch(watch)
+
+    def use_artifact(self, artifact_or_name, type=None, aliases=None):
+        """ Declare an artifact as an input to a run, call `download` or `file` on \
+        the returned object to get the contents locally.
+
+        Args:
+            artifact_or_name (str or Artifact): An artifact name.
+            May be prefixed with entity/project. Valid names
+                can be in the following forms:
+                    name:version
+                    name:alias
+                    digest
+                You can also pass an Artifact object created by calling `wandb.Artifact`
+            type (str, optional): The type of artifact to use.
+            aliases (list, optional): Aliases to apply to this artifact
+        Returns:
+            A :obj:`Artifact` object.
+        """
+        r = self._run_obj
+        api = internal.Api(default_settings={"entity": r.entity, "project": r.project})
+        api.set_current_run_id(self.id)
+
+        if isinstance(artifact_or_name, str):
+            name = artifact_or_name
+            if type is None:
+                raise ValueError("type required")
+            public_api = public.Api()
+            artifact = public_api.artifact(type=type, name=name)
+            if type is not None and type != artifact.type:
+                raise ValueError(
+                    "Supplied type {} does not match type {} of artifact {}".format(
+                        type, artifact.type, artifact.name
+                    )
+                )
+            api.use_artifact(artifact.id)
+            return artifact
+        else:
+            artifact = artifact_or_name
+            if type is not None:
+                raise ValueError("cannot specify type when passing Artifact object")
+            if isinstance(aliases, str):
+                aliases = [aliases]
+            if isinstance(artifact_or_name, wandb.Artifact):
+                artifact.finalize()
+                self._backend.interface.send_artifact(
+                    self, artifact, aliases, is_user_created=True, use_after_commit=True
+                )
+                return artifact
+            elif isinstance(artifact, public.Artifact):
+                api.use_artifact(artifact.id)
+                return artifact
+            else:
+                raise ValueError(
+                    'You must pass an artifact name (e.g. "pedestrian-dataset:v1"), an instance of wandb.Artifact, or wandb.Api().artifact() to use_artifact'  # noqa: E501
+                )
+
+    def log_artifact(self, artifact_or_path, name=None, type=None, aliases=None):
+        """ Declare an artifact as output of a run.
+
+        Args:
+            artifact_or_path (str or Artifact): A path to the contents of this artifact,
+                can be in the following forms:
+                    /local/directory
+                    /local/directory/file.txt
+                    s3://bucket/path
+                You can also pass an Artifact object created by calling
+                `wandb.Artifact`.
+            name (str, optional): An artifact name. May be prefixed with entity/project.
+                Valid names can be in the following forms:
+                    name:version
+                    name:alias
+                    digest
+                This will default to the basename of the path prepended with the current
+                run id  if not specified.
+            type (str): The type of artifact to log, examples include "dataset", "model"
+            aliases (list, optional): Aliases to apply to this artifact,
+                defaults to ["latest"]
+        Returns:
+            A :obj:`Artifact` object.
+        """
+        aliases = aliases or ["latest"]
+        if isinstance(artifact_or_path, str):
+            if name is None:
+                name = "run-%s-%s" % (self.id, os.path.basename(artifact_or_path))
+            artifact = wandb.Artifact(name, type)
+            if os.path.isfile(artifact_or_path):
+                artifact.add_file(artifact_or_path)
+            elif os.path.isdir(artifact_or_path):
+                artifact.add_dir(artifact_or_path)
+            elif "://" in artifact_or_path:
+                artifact.add_reference(artifact_or_path)
+            else:
+                raise ValueError(
+                    "path must be a file, directory or external"
+                    "reference like s3://bucket/path"
+                )
+        else:
+            artifact = artifact_or_path
+        if not isinstance(artifact, wandb.Artifact):
+            raise ValueError(
+                "You must pass an instance of wandb.Artifact or a "
+                "valid file path to log_artifact"
+            )
+        if isinstance(aliases, str):
+            aliases = [aliases]
+        artifact.finalize()
+        self._backend.interface.send_artifact(self, artifact, aliases)
+        return artifact
