@@ -21,15 +21,17 @@ import collections
 import configparser
 import copy
 import datetime
+import getpass
 import logging
 import os
 import platform
+import socket
 import sys
 
 import shortuuid  # type: ignore
 import six
 import wandb
-from wandb import env
+from wandb import jupyter
 from wandb.internal import git_repo
 
 if wandb.TYPE_CHECKING:  # type: ignore
@@ -89,6 +91,10 @@ env_settings = dict(
     console=None,
     config_paths=None,
     run_id=None,
+    notebook_name=None,
+    host=None,
+    username=None,
+    disable_code=None,
     run_name="WANDB_NAME",
     run_notes="WANDB_NOTES",
     run_tags="WANDB_TAGS",
@@ -259,6 +265,14 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
         _internal_check_process=8,
         _disable_meta=None,
         _disable_stats=None,
+        _jupyter_path=None,
+        _jupyter_name=None,
+        _jupyter_root=None,
+        _executable=None,
+        _cuda=None,
+        _args=None,
+        _os=None,
+        _python=None,
     ):
         kwargs = locals()
         object.__setattr__(self, "_masked_keys", set(["self", "_frozen"]))
@@ -391,11 +405,41 @@ class Settings(six.with_metaclass(CantTouchThis, object)):
         # For code saving, only allow env var override if value from server is true, or
         # if no preference was specified.
         if (self.save_code is True or self.save_code is None) and os.getenv(
-            env.SAVE_CODE
+            wandb.env.SAVE_CODE
         ) is not None:
-            u["save_code"] = env.should_save_code()
+            u["save_code"] = wandb.env.should_save_code()
+
+        if self.jupyter:
+            meta = jupyter.notebook_metadata()
+            u["_jupyter_path"] = meta.get("path")
+            u["_jupyter_name"] = meta.get("name")
+            u["_jupyter_root"] = meta.get("root")
+
+        # host and username are populated by env_settings above if their env
+        # vars exist -- but if they don't, we'll fill them in here
+        if not self.host:
+            u["host"] = socket.gethostname()
+
+        if not self.username:
+            try:
+                u["username"] = getpass.getuser()
+            except KeyError:
+                # getuser() could raise KeyError in restricted environments like
+                # chroot jails or docker containers.  Return user id in these cases.
+                u["username"] = str(os.getuid())
+
+        u["_executable"] = sys.executable
+
+        u["docker"] = wandb.env.get_docker()
+
+        # TODO: we should use the cuda library to collect this
+        if os.path.exists("/usr/local/cuda/version.txt"):
+            with open("/usr/local/cuda/version.txt") as f:
+                u["_cuda"] = f.read().split(" ")[-1].strip()
+        u["_args"] = sys.argv[1:]
+        u["_os"] = platform.platform(aliased=True)
+        u["_python"] = platform.python_version()
 
-        u["disable_code"] = os.getenv(env.DISABLE_CODE)
         self.update(u)
 
         # If the settings say to save code, and there's not already a program file,
